// This file is part of cxsd, copyright (c) 2015-2016 BusFaster Ltd.
// Released under the MIT license, see LICENSE.
"use strict";
var cmd = require('commander');
var cget_1 = require('cget');
var Namespace_1 = require('./xsd/Namespace');
var Loader_1 = require('./xsd/Loader');
var Exporter_1 = require('./xsd/Exporter');
var schema = require('./schema');
var AddImports_1 = require('./schema/transform/AddImports');
var Sanitize_1 = require('./schema/transform/Sanitize');
var ListImports_1 = require('./schema/transform/ListImports');
(cmd.version(require('../package.json').version)
    .arguments('<url>')
    .description('XSD download and conversion tool')
    .option('-H, --force-host <host>', 'Fetch all xsd files from <host>\n    (original host is passed in GET parameter "host")')
    .option('-P, --force-port <port>', 'Connect to <port> when using --force-host')
    .option('-c, --cache-xsd <path>', 'Cache downloaded XSD filed under <path>')
    .option('-t, --out-ts <path>', 'Output TypeScript definitions under <path>')
    .option('-j, --out-js <path>', 'Output JavaScript modules under <path>')
    .action(handleConvert)
    .parse(process.argv));
function handleConvert(urlRemote, opts) {
    var xmlSpace = Namespace_1.Namespace.register('http://www.w3.org/XML/1998/namespace', 'http://www.w3.org/2001/xml.xsd', 'xml');
    var fetchOptions = {};
    if (opts['forceHost']) {
        fetchOptions.forceHost = opts['forceHost'];
        if (opts['forcePort'])
            fetchOptions.forcePort = opts['forcePort'];
        cget_1.Cache.patchRequest();
    }
    var jsCache = new cget_1.Cache(opts['outJs'] || 'cache/js', '_index.js');
    var tsCache = new cget_1.Cache(opts['outTs'] || 'cache/js', '_index.d.ts');
    var loader = new Loader_1.Loader(fetchOptions);
    loader.import(urlRemote).then(function (namespace) {
        try {
            Exporter_1.exportNamespace(Namespace_1.PrimitiveSpace.get());
            Exporter_1.exportNamespace(xmlSpace);
            var spec = Exporter_1.exportNamespace(namespace);
            new AddImports_1.AddImports(spec).exec().then(function () {
                return new Sanitize_1.Sanitize(spec).exec();
            }).then(function (sanitize) {
                return sanitize.finish();
            }).then(function () {
                return new ListImports_1.ListImports(spec).exec();
            }).then(function () {
                return new schema.exporter.JS(spec, jsCache).exec();
            }).then(function () {
                return new schema.exporter.TS(spec, tsCache).exec();
            });
        }
        catch (err) {
            console.log(err);
            console.log(err.stack);
        }
    });
}
