// This file is part of cxsd, copyright (c) 2016 BusFaster Ltd.
// Released under the MIT license, see LICENSE.
"use strict";
(function (TypeState) {
    TypeState[TypeState["anonymous"] = 0] = "anonymous";
    TypeState[TypeState["exported"] = 1] = "exported";
})(exports.TypeState || (exports.TypeState = {}));
var TypeState = exports.TypeState;
var Namespace = (function () {
    function Namespace(id, name, short) {
        /** All types used in the document. */
        this.typeList = [];
        this.typeStateList = [];
        /** Short names used to reference other namespaces in schemas defining this namespace. */
        this.shortNameTbl = {};
        this.id = id;
        this.name = name;
        this.short = short;
    }
    Namespace.prototype.addRef = function (shortName, namespace) {
        var id = namespace.id;
        if (!this.shortNameTbl[id])
            this.shortNameTbl[id] = [];
        this.shortNameTbl[id].push(shortName);
    };
    Namespace.prototype.getShortRef = function (id) {
        var nameList = this.shortNameTbl[id];
        if (nameList && nameList.length)
            return (nameList[0]);
        else
            return (null);
    };
    Namespace.prototype.getUsedImportTbl = function () {
        var importTbl = this.importTbl;
        if (!importTbl) {
            importTbl = {};
            if (this.importTypeNameTbl) {
                for (var _i = 0, _a = Object.keys(this.importTypeNameTbl); _i < _a.length; _i++) {
                    var key = _a[_i];
                    var id = +key;
                    var short = this.getShortRef(id);
                    importTbl[this.getShortRef(id)] = Namespace.list[id];
                }
                this.importTbl = importTbl;
            }
        }
        return (importTbl);
    };
    Namespace.prototype.getUsedImportList = function () {
        if (this.importTypeNameTbl) {
            var importTbl = this.getUsedImportTbl();
            return (Object.keys(importTbl).map(function (shortName) {
                return importTbl[shortName];
            }));
        }
        else {
            return (Object.keys(this.shortNameTbl).map(function (id) {
                return Namespace.list[+id];
            }));
        }
    };
    Namespace.prototype.addType = function (type) {
        var id = type.surrogateKey;
        this.typeList[id] = type;
        this.typeStateList[id] = TypeState.anonymous;
        type.namespace = this;
    };
    Namespace.prototype.exportType = function (type) {
        this.typeStateList[type.surrogateKey] = TypeState.exported;
    };
    Namespace.register = function (id, name, short) {
        var namespace = Namespace.list[id];
        if (!namespace) {
            namespace = new Namespace(id, name, short);
            Namespace.list[id] = namespace;
        }
        else if (!namespace.short)
            namespace.short = short;
        return (namespace);
    };
    /** Internal list of namespaces indexed by a surrogate key. */
    Namespace.list = [];
    return Namespace;
}());
exports.Namespace = Namespace;
