// This file is part of cxsd, copyright (c) 2016 BusFaster Ltd.
// Released under the MIT license, see LICENSE.
"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var Exporter_1 = require('./Exporter');
var Namespace_1 = require('../Namespace');
var Type_1 = require('../Type');
var JS = (function (_super) {
    __extends(JS, _super);
    function JS() {
        _super.apply(this, arguments);
        this.construct = JS;
    }
    JS.prototype.writeImport = function (shortName, relativePath, absolutePath) {
        return ('var ' +
            shortName +
            ' = require(' +
            "'" + relativePath + "'" +
            ');');
    };
    JS.prototype.writeMember = function (member, typeNumTbl) {
        var name = member.safeName;
        if (member.name != name)
            name += ':' + member.name;
        var flags = 0;
        if (member.min < 1)
            flags |= Type_1.Type.optionalFlag;
        if (member.max > 1)
            flags |= Type_1.Type.arrayFlag;
        var memberTypeList = member.typeList.map(function (memberType) {
            return typeNumTbl[memberType.surrogateKey];
        });
        return ('[' +
            "'" + name + "', " +
            flags + ', ' +
            '[' + memberTypeList.join(', ') + ']' +
            ']');
    };
    JS.prototype.writeType = function (type, typeNumTbl) {
        var childSpecList = [];
        var attributeSpecList = [];
        if (type.childList) {
            for (var _i = 0, _a = type.childList; _i < _a.length; _i++) {
                var member = _a[_i];
                childSpecList.push(this.writeMember(member, typeNumTbl));
            }
        }
        if (type.attributeList) {
            for (var _b = 0, _c = type.attributeList; _b < _c.length; _b++) {
                var member = _c[_b];
                attributeSpecList.push(this.writeMember(member, typeNumTbl));
            }
        }
        var parentNum = type.parent ? typeNumTbl[type.parent.surrogateKey] : 0;
        return ('\n\t[' +
            parentNum + ', ' +
            '[' + childSpecList.join(', ') + '], ' +
            '[' + attributeSpecList.join(', ') + ']' +
            ']');
    };
    /** Output namespace contents to the given cache key. */
    JS.prototype.writeContents = function () {
        var doc = this.doc;
        var namespace = doc.namespace;
        var typeNumTbl = {};
        var typeNum = 1;
        var importTbl = namespace.getUsedImportTbl();
        var importNameList = Object.keys(importTbl);
        var importSpecList = [];
        for (var _i = 0, importNameList_1 = importNameList; _i < importNameList_1.length; _i++) {
            var importName = importNameList_1[_i];
            var otherNamespaceId = importTbl[importName].id;
            var importTypeNameTbl = namespace.importTypeNameTbl[otherNamespaceId];
            var importTypeNameList = [];
            if (importTypeNameTbl) {
                for (var _a = 0, _b = Object.keys(importTypeNameTbl).sort(); _a < _b.length; _a++) {
                    var name = _b[_a];
                    var type = importTypeNameTbl[name];
                    importTypeNameList.push("'" + type.safeName + "'");
                    typeNumTbl[type.surrogateKey] = typeNum++;
                }
            }
            importSpecList.push('\n\t' + '[' + importName + ', [' +
                importTypeNameList.join(', ') +
                ']]');
        }
        var exportedTypeList = [];
        var hiddenTypeList = [];
        var typeSpecList = [];
        for (var _c = 0, _d = namespace.typeList; _c < _d.length; _c++) {
            var type = _d[_c];
            if (!type)
                continue;
            var isExported = (namespace.typeStateList[type.surrogateKey] == Namespace_1.TypeState.exported);
            if (isExported)
                exportedTypeList.push(type);
            else
                hiddenTypeList.push(type);
        }
        exportedTypeList.sort(function (a, b) { return a.safeName.localeCompare(b.safeName); });
        hiddenTypeList.sort(function (a, b) { return a.safeName.localeCompare(b.safeName); });
        var typeList = exportedTypeList.concat(hiddenTypeList);
        for (var _e = 0, typeList_1 = typeList; _e < typeList_1.length; _e++) {
            var type = typeList_1[_e];
            typeNumTbl[type.surrogateKey] = typeNum++;
        }
        var parentNum;
        typeSpecList.push(this.writeType(namespace.doc, typeNumTbl));
        for (var _f = 0, typeList_2 = typeList; _f < typeList_2.length; _f++) {
            var type = typeList_2[_f];
            typeSpecList.push(this.writeType(type, typeNumTbl));
        }
        var exportSpecList = [];
        for (var _g = 0, exportedTypeList_1 = exportedTypeList; _g < exportedTypeList_1.length; _g++) {
            var type = exportedTypeList_1[_g];
            name = type.safeName;
            if (type.name && type.name != name)
                name += ':' + type.name;
            exportSpecList.push('\n\t' + "'" + name + "'");
        }
        return ([].concat([
            'var cxml = require("cxml");',
        ], this.writeHeader(), [
            '',
            'cxml.register(' +
                "'" + namespace.name + "', " +
                'exports, ' +
                '[' + importSpecList.join(',') + '\n], ' +
                '[' + exportSpecList.join(',') + '\n], ' +
                '[' + typeSpecList.join(',') + '\n]' +
                ');'
        ]).join('\n'));
    };
    JS.prototype.getOutName = function (name) {
        return (name + '.js');
    };
    return JS;
}(Exporter_1.Exporter));
exports.JS = JS;
