// This file is part of cxsd, copyright (c) 2016 BusFaster Ltd.
// Released under the MIT license, see LICENSE.
"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var Transform_1 = require('./Transform');
function capitalize(match, initial) {
    return (initial.toUpperCase());
}
function sanitizeName(name) {
    var reserved = {
        'constructor': true
    };
    name = name
        .replace(/-([a-z])/, capitalize)
        .replace(/[^_0-9A-Za-z]/g, '')
        .replace(/^[^A-Za-z]+/, '');
    if (reserved.hasOwnProperty(name))
        name = '_' + name;
    return (name);
}
var Sanitize = (function (_super) {
    __extends(Sanitize, _super);
    function Sanitize() {
        _super.apply(this, arguments);
        this.state = {
            pendingAnonTbl: {},
            pendingAnonList: [],
            typeListList: []
        };
        this.construct = Sanitize;
    }
    Sanitize.prototype.prepare = function () {
        var typeList = this.namespace.typeList.filter(function (type) { return !!type; });
        var type;
        for (var _i = 0, typeList_1 = typeList; _i < typeList_1.length; _i++) {
            type = typeList_1[_i];
            type.buildMemberTbl();
        }
        this.visitType(this.doc);
        for (var _a = 0, typeList_2 = typeList; _a < typeList_2.length; _a++) {
            type = typeList_2[_a];
            this.visitType(type);
        }
        this.state.typeListList.push(typeList);
        return (true);
    };
    Sanitize.prototype.renameDuplicates = function (typeList) {
        // TODO: handle collisions between names of types and members of doc.
        // Sort types by sanitized name and duplicates by original name
        // (missing original names sorted after existing original names).
        // TODO: merge types with identical contents.
        typeList = typeList.sort(function (a, b) {
            return a.safeName.localeCompare(b.safeName) ||
                +!!b.name - +!!a.name ||
                (a.name && a.name.localeCompare(b.name));
        });
        // Add numeric suffix to duplicate names.
        var name = '';
        var suffix = 2;
        for (var _i = 0, typeList_3 = typeList; _i < typeList_3.length; _i++) {
            var type = typeList_3[_i];
            if (type.safeName == name) {
                type.safeName += '_' + (suffix++);
            }
            else {
                name = type.safeName;
                suffix = 2;
            }
        }
    };
    Sanitize.prototype.finish = function () {
        for (var _i = 0, _a = Object.keys(this.state.pendingAnonTbl); _i < _a.length; _i++) {
            var key = _a[_i];
            var spec = this.state.pendingAnonTbl[key];
            if (spec) {
                for (var _b = 0, _c = spec.memberTypeList; _b < _c.length; _b++) {
                    var memberType = _c[_b];
                    if (memberType.containingType.safeName)
                        this.addNameToType(memberType);
                }
            }
        }
        for (var _d = 0, _e = this.state.pendingAnonList; _d < _e.length; _d++) {
            var type = _e[_d];
            if (!type.safeName)
                type.safeName = 'Type';
        }
        for (var _f = 0, _g = this.state.typeListList; _f < _g.length; _f++) {
            var typeList = _g[_f];
            this.renameDuplicates(typeList);
        }
    };
    Sanitize.prototype.visitType = function (type) {
        var memberList = [];
        var member;
        var other;
        var otherMember;
        var iter;
        if (type.name)
            type.safeName = sanitizeName(type.name);
        else
            this.state.pendingAnonList.push(type);
        if (type.attributeList) {
            for (var _i = 0, _a = type.attributeList; _i < _a.length; _i++) {
                member = _a[_i];
                // Add a $ prefix to attributes of this type
                // conflicting with children of this or parent types.
                other = type;
                iter = 100;
                while (other && --iter) {
                    otherMember = other.childTbl[member.name];
                    if (otherMember) {
                        member.prefix = '$';
                        break;
                    }
                    other = other.parent;
                }
                memberList.push(member);
            }
        }
        if (type.childList) {
            for (var _b = 0, _c = type.childList; _b < _c.length; _b++) {
                member = _c[_b];
                // Add a $ prefix to attributes of parent types
                // conflicting with children of this type.
                other = type;
                iter = 100;
                while ((other = other.parent) && --iter) {
                    otherMember = other.attributeTbl[member.name];
                    if (otherMember && !otherMember.prefix) {
                        otherMember.prefix = '$';
                        if (otherMember.safeName)
                            otherMember.safeName = otherMember.prefix + otherMember.safeName;
                    }
                }
                // Ensure maximum allowed occurrence count is no less than in parent types,
                // because overriding a parent class member with a different type
                // (array vs non-array) doesn't compile.
                if (member.max < 2) {
                    other = type;
                    iter = 100;
                    // TODO: Topologically sort dependencies to start processing from root types,
                    // to avoid continuing search after one parent with a matching member is found.
                    while ((other = other.parent) && --iter) {
                        otherMember = other.childTbl[member.name];
                        if (otherMember && otherMember.max > member.max) {
                            member.max = otherMember.max;
                            if (member.max > 1)
                                break;
                        }
                    }
                }
                memberList.push(member);
            }
        }
        // Add names to any unnamed types of members, based on the member name.
        for (var _d = 0, memberList_1 = memberList; _d < memberList_1.length; _d++) {
            var member = memberList_1[_d];
            // TODO: Detect duplicate names from other namespaces and prefix them.
            member.safeName = (member.prefix || '') + sanitizeName(member.name);
            this.addNameToMemberTypes(type, member);
        }
    };
    Sanitize.prototype.addNameToType = function (type) {
        var containingType = type.containingType;
        var containingMember = type.containingMember;
        if (containingMember) {
            type.namespace = containingMember.namespace;
            type.safeName = (containingType ? containingType.safeName : '') + (containingMember.safeName || '').replace(/^([a-z])/, capitalize) + 'Type';
        }
        var spec = this.state.pendingAnonTbl[type.surrogateKey];
        if (spec) {
            for (var _i = 0, _a = spec.memberTypeList; _i < _a.length; _i++) {
                var memberType = _a[_i];
                this.addNameToType(memberType);
            }
            this.state.pendingAnonTbl[type.surrogateKey] = null;
        }
    };
    Sanitize.prototype.addNameToMemberTypes = function (type, member) {
        for (var _i = 0, _a = member.typeList; _i < _a.length; _i++) {
            var memberType = _a[_i];
            if (!memberType.safeName && memberType.namespace == this.namespace) {
                if (memberType.containingType) {
                    if (memberType.containingType.safeName)
                        this.addNameToType(memberType);
                    else {
                        var spec = this.state.pendingAnonTbl[memberType.containingType.surrogateKey];
                        if (!spec) {
                            spec = { type: memberType.containingType, memberTypeList: [] };
                            this.state.pendingAnonTbl[memberType.containingType.surrogateKey] = spec;
                        }
                        spec.memberTypeList.push(memberType);
                    }
                }
                else if (memberType.containingMember) {
                    if (memberType.containingMember.safeName)
                        this.addNameToType(memberType);
                }
            }
        }
    };
    return Sanitize;
}(Transform_1.Transform));
exports.Sanitize = Sanitize;
