import { Loader } from './Loader';
import { Source } from './Source';
import { Scope } from './Scope';
/** XML namespace, binding it to syntax definitions. */
export declare class Namespace {
    constructor();
    /** Initialize names and addresses. Can be called multiple times. */
    init(name: string, url?: string, short?: string): this;
    static sanitize(name: string): string;
    /** Globally register a namespace, to attach handlers to it. */
    static register(name: string, url?: string, short?: string): Namespace;
    /** Find a globally registered namespace. */
    static lookup(name: string): Namespace;
    static byId(id: number): Namespace;
    /** Load and parse the main schema file for this namespace. */
    importSchema(loader: Loader, urlRemote?: string): Source;
    /** Update final address of schema file if HTTP request was redirected. */
    updateUrl(urlOld: string, urlNew: string): void;
    /** Fetch the root scope with published attributes, groups, elements... */
    getScope(): Scope;
    /** @return List of all source files potentially contributing to this namespace. */
    getSourceList(): Source[];
    /** Internal list of namespaces indexed by a surrogate key. */
    private static list;
    /** Table of namespaces based on their names and addresses. */
    private static tbl;
    /** Surrogate key, used internally as a unique namespace ID. */
    id: number;
    /** URL address identifying the namespace (not used to download anything). */
    name: string;
    /** URL address where main schema file was downloaded. */
    url: string;
    /** Example short name for this namespace. */
    short: string;
    /** List of all source files potentially contributing to this namespace. */
    private sourceList;
    /** Source files potentially contributing to this namespace. */
    private sourceTbl;
    /** Global scope where exported members will be published. */
    private scope;
}
/** Special singleton namespace that contains primitive types. */
export declare class PrimitiveSpace extends Namespace {
    /** Construct a table mapping XSD primitive type names to their handler classes. */
    constructor();
    static get(): PrimitiveSpace;
    static getScope(): Scope;
    static instance: PrimitiveSpace;
    name: string;
    short: string;
}
