// This file is part of cxsd, copyright (c) 2015-2016 BusFaster Ltd.
// Released under the MIT license, see LICENSE.
"use strict";
var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var Source_1 = require('./Source');
var Scope_1 = require('./Scope');
var State_1 = require('./State');
var QName_1 = require('./QName');
var Primitive_1 = require('./types/Primitive');
var schema = require('../schema');
/** XML namespace, binding it to syntax definitions. */
var Namespace = (function () {
    function Namespace() {
        /** List of all source files potentially contributing to this namespace. */
        this.sourceList = [];
        /** Source files potentially contributing to this namespace. */
        this.sourceTbl = {};
        var id = Namespace.list.length;
        Namespace.list[id] = this;
        this.id = id;
        this.scope = new Scope_1.Scope(this instanceof PrimitiveSpace ? null : PrimitiveSpace.getScope(), this);
    }
    /** Initialize names and addresses. Can be called multiple times. */
    Namespace.prototype.init = function (name, url, short) {
        name = Namespace.sanitize(name);
        if (name) {
            if (!this.name)
                this.name = name;
            Namespace.tbl[name] = this;
        }
        if (url) {
            if (!this.url)
                this.url = url;
            Namespace.tbl[url] = this;
        }
        if (short) {
            if (!this.short)
                this.short = short;
            Namespace.tbl[short] = this;
        }
        return (this);
    };
    Namespace.sanitize = function (name) {
        return (name && name.replace(/\/+$/, ''));
    };
    /** Globally register a namespace, to attach handlers to it. */
    Namespace.register = function (name, url, short) {
        name = Namespace.sanitize(name);
        var namespace = Namespace.tbl[name] || Namespace.tbl[url];
        if (!namespace) {
            namespace = new Namespace();
        }
        return (namespace.init(name, url, short));
    };
    /** Find a globally registered namespace. */
    Namespace.lookup = function (name) {
        name = Namespace.sanitize(name);
        return (Namespace.tbl[name]);
    };
    Namespace.byId = function (id) {
        return (Namespace.list[id]);
    };
    /** Load and parse the main schema file for this namespace. */
    Namespace.prototype.importSchema = function (loader, urlRemote) {
        var source = loader.importFile(this, urlRemote || this.url);
        if (!this.sourceTbl[source.id]) {
            this.sourceTbl[source.id] = source;
            this.sourceList.push(source);
        }
        return (source);
    };
    /** Update final address of schema file if HTTP request was redirected. */
    Namespace.prototype.updateUrl = function (urlOld, urlNew) {
        if (!this.url || this.url == urlOld)
            this.url = urlNew;
    };
    /** Fetch the root scope with published attributes, groups, elements... */
    Namespace.prototype.getScope = function () { return (this.scope); };
    /** @return List of all source files potentially contributing to this namespace. */
    Namespace.prototype.getSourceList = function () {
        return (this.sourceList);
    };
    /** Internal list of namespaces indexed by a surrogate key. */
    Namespace.list = [];
    /** Table of namespaces based on their names and addresses. */
    Namespace.tbl = {};
    return Namespace;
}());
exports.Namespace = Namespace;
/** Special singleton namespace that contains primitive types. */
var PrimitiveSpace = (function (_super) {
    __extends(PrimitiveSpace, _super);
    /** Construct a table mapping XSD primitive type names to their handler classes. */
    function PrimitiveSpace() {
        _super.call(this);
        this.name = 'xml-primitives';
        this.short = 'Primitive';
        var spec = [
            [
                'boolean',
                'boolean'
            ], [
                'date dateTime duration time ' +
                    'byte decimal double float int integer long negativeInteger nonNegativeInteger nonPositiveInteger positiveInteger short unsignedLong unsignedInt unsignedShort unsignedByte',
                'number'
            ], [
                'anyURI ID IDREF IDREFS language NCName NMTOKEN NMTOKENS normalizedString QName string token',
                'string'
            ]
        ];
        var scope = this.getScope();
        var source = new Source_1.Source(this, '');
        var state = new State_1.State(null, null, source);
        state.setScope(scope);
        schema.Namespace.register(this.id, this.name, this.short).isPrimitiveSpace = true;
        for (var _i = 0, spec_1 = spec; _i < spec_1.length; _i++) {
            var typeSpec = spec_1[_i];
            var type = new Primitive_1.Primitive(null);
            type.name = typeSpec[1];
            type.init(new State_1.State(state, null));
            var outType = type.getOutType();
            outType.literalType = outType;
            outType.safeName = type.name;
            for (var _a = 0, _b = typeSpec[0].split(' '); _a < _b.length; _a++) {
                var name = _b[_a];
                scope.add(new QName_1.QName().parsePrimitive(name, this), 'type', type, 1, 1);
            }
        }
    }
    PrimitiveSpace.get = function () {
        var instance = PrimitiveSpace.instance;
        if (!instance) {
            instance = new PrimitiveSpace();
            PrimitiveSpace.instance = instance;
        }
        return (instance);
    };
    PrimitiveSpace.getScope = function () {
        return (PrimitiveSpace.get().getScope());
    };
    return PrimitiveSpace;
}(Namespace));
exports.PrimitiveSpace = PrimitiveSpace;
