// This file is part of cxsd, copyright (c) 2015-2016 BusFaster Ltd.
// Released under the MIT license, see LICENSE.
"use strict";
var url = require('url');
var Namespace_1 = require('./Namespace');
/** Details of a single XSD source file. */
var Source = (function () {
    function Source(targetNamespace, urlRemote) {
        /** Table of namespace shorthand references (xmlns:...) used in this file. */
        this.namespaceRefTbl = {};
        var id = Source.list.length;
        this.id = id;
        this.targetNamespace = targetNamespace;
        this.url = urlRemote;
        this.urlOriginal = urlRemote;
        Source.list[id] = this;
    }
    /** Called by the parser, converts XSD attributes describing the schema into references to internal objects. */
    Source.prototype.parse = function (attrTbl) {
        // Unqualified tags are assumed to be in the default namespace.
        // For the schema file itself, it should be http://www.w3.org/2001/XMLSchema
        if (attrTbl['xmlns']) {
            this.defaultNamespace = Namespace_1.Namespace.register(attrTbl['xmlns']);
        }
        // Everything defined in the current file belongs to the target namespace by default.
        if (attrTbl['targetnamespace']) {
            this.targetNamespace.init(attrTbl['targetnamespace']);
        }
        // Read the current file's preferred shorthand codes for other XML namespaces.
        for (var _i = 0, _a = Object.keys(attrTbl); _i < _a.length; _i++) {
            var attr = _a[_i];
            if (attr.match(/^xmlns:/i)) {
                var short = attr.substr(attr.indexOf(':') + 1);
                this.namespaceRefTbl[short] = Namespace_1.Namespace.register(attrTbl[attr], null, short);
            }
        }
    };
    /** Find a namespace according to its full name or the short name as used in this source file. */
    Source.prototype.lookupNamespace = function (ref) {
        return (this.namespaceRefTbl[ref] || Namespace_1.Namespace.lookup(ref));
    };
    /** Resolve a possible relative URL in the context of this source file. */
    Source.prototype.urlResolve = function (urlRemote) {
        return (url.resolve(this.targetNamespace.url, urlRemote));
    };
    /** Update current remote address, in case the previous address got redirected. */
    Source.prototype.updateUrl = function (urlRemote) { this.url = urlRemote; };
    Source.prototype.getNamespaceRefs = function () {
        return (this.namespaceRefTbl);
    };
    /** Internal list of source files indexed by a surrogate key. */
    Source.list = [];
    return Source;
}());
exports.Source = Source;
