// This file is part of cxsd, copyright (c) 2015-2016 BusFaster Ltd.
// Released under the MIT license, see LICENSE.
"use strict";
var Namespace_1 = require('../Namespace');
var QName_1 = require('../QName');
/** Common handler base class for all schema tags. */
var Base = (function () {
    function Base(state) {
        if (!state)
            return;
        this.scope = state.getScope();
        this.lineNumber = state.stateStatic.getLineNumber();
        this.bytePos = state.stateStatic.getBytePos();
    }
    /** Hook, runs after opening tag. */
    Base.prototype.init = function (state) { };
    /** Hook, runs for text content. */
    Base.prototype.addText = function (state, text) { };
    /** Hook, runs after closing tag. */
    Base.prototype.loaded = function (state) { };
    /** Hook, runs after all dependencies have been loaded. */
    Base.prototype.resolve = function (state) { };
    /** Add this named tag to scope, listed under given type.
      * Optionally set number of allowed occurrences, for optional elements, sequences etc.
        * @return fully qualified name. */
    Base.prototype.define = function (state, type, min, max, scope) {
        if (min === void 0) { min = 1; }
        if (max === void 0) { max = 1; }
        if (!this.name)
            return (null);
        var qName = new QName_1.QName(this.name, state.source);
        (scope || this.scope).addToParent(qName, type, this, min, max);
        return (qName);
    };
    Base.prototype.getScope = function () { return (this.scope); };
    Base.getNamespace = function () {
        if (!Base.namespace) {
            Base.namespace = Namespace_1.Namespace.register('http://www.w3.org/2001/XMLSchema', 'http://www.w3.org/2009/XMLSchema/XMLSchema.xsd', 'xsd');
        }
        return (Base.namespace);
    };
    /** Returns other classes allowed as children. */
    Base.mayContain = function () { return ([]); };
    return Base;
}());
exports.Base = Base;
